@@ -1,7 +1,7 @@
 #' Check names to comply to specific rules
 #'
 #' @description
-#' Similar to \code{\link{checkNamed}} but you can pass the names directly.
+#' Performs various checks on character vectors, usually names.
 #'
 #' @templateVar fn Named
 #' @param x [\code{character} || \code{NULL}]\cr
@@ -13,6 +13,7 @@
 #'  \item{named:}{Checks \code{x} for regular names which excludes names to be \code{NA} or empty (\code{""}).}
 #'  \item{unique:}{Performs checks like with \dQuote{named} and additionally tests for non-duplicated names.}
 #'  \item{strict:}{Performs checks like with \dQuote{unique} and additionally fails for names with UTF-8 characters and names which do not comply to R's variable name restrictions. As regular expression, this is \dQuote{^[.]*[a-zA-Z]+[a-zA-Z0-9._]*$}.}
+#'  \item{ids:}{Same as \dQuote{strict}, but does not enforce uniqueness.}
 #'  }
 #'  Note that for zero-length \code{x}, all these name checks evaluate to \code{TRUE}.
 #' @param subset.of [\code{character}]\cr
@@ -27,6 +28,10 @@
 #' @param identical.to [\code{character}]\cr
 #'  Names provided in \code{x} must be identical to the vector \code{identical.to}.
 #'  Use this argument instead of \code{permutation.of} if the order of the names is relevant.
+#' @param disjunct.from [\code{character}]\cr
+#'  Names provided in \code{x} must may not be present in the vector \code{identical.to}.
+#' @param what [\code{character(1)}]\cr
+#'  Type of name vector to check, e.g. \dQuote{names} (default), \dQuote{colnames} or \dQuote{rownames}.
 #' @template checker
 #' @useDynLib checkmate c_check_names
 #' @family attributes
@@ -39,12 +44,12 @@
 #'
 #' cn = c("Species", "Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width")
 #' assertNames(names(iris), permutation.of = cn)
-checkNames = function(x, type = "named", subset.of = NULL, must.include = NULL, permutation.of = NULL, identical.to = NULL) {
-  .Call(c_check_names, x, type) %and%
-    checkNamesCmp(x, subset.of, must.include, permutation.of, identical.to)
+checkNames = function(x, type = "named", subset.of = NULL, must.include = NULL, permutation.of = NULL, identical.to = NULL, disjunct.from = NULL, what = "names") {
+  .Call(c_check_names, x, type, what) %and%
+    checkNamesCmp(x, subset.of, must.include, permutation.of, identical.to, disjunct.from)
 }
 
-checkNamesCmp = function(x, subset.of, must.include, permutation.of, identical.to) {
+checkNamesCmp = function(x, subset.of, must.include, permutation.of, identical.to, disjunct.from) {
   if (!is.null(subset.of)) {
     qassert(subset.of, "S")
     if (anyMissing(match(x, subset.of)))
@@ -65,6 +70,10 @@ checkNamesCmp = function(x, subset.of, must.include, permutation.of, identical.t
     if (!identical(x, identical.to))
       return(sprintf("Must be a identical to (%s)", paste0(identical.to, collapse = ",")))
   }
+  if (!is.null(disjunct.from)) {
+    if (any(x %in% disjunct.from))
+      return(sprintf("Must be disjunct from (%s)", paste0(disjunct.from, collapse = ",")))
+  }
   return(TRUE)
 }
 
@@ -76,7 +85,7 @@ check_names = checkNames
 #' @include makeAssertion.R
 #' @template assert
 #' @rdname checkNames
-assertNames = makeAssertionFunction(checkNames)
+assertNames = makeAssertionFunction(checkNames, use.namespace = FALSE)
 
 #' @export
 #' @rdname checkNames
@@ -95,4 +104,4 @@ test_names = testNames
 #' @include makeExpectation.R
 #' @template expect
 #' @rdname checkNames
-expect_names = makeExpectationFunction(checkNames)
+expect_names = makeExpectationFunction(checkNames, use.namespace = FALSE)
