@@ -1,11 +1,13 @@
 #include <ctype.h>
 #include <string.h>
+#include "backports.h"
 #include "checks.h"
-#include "is_integerish.h"
+#include "integerish.h"
+#include "is_sorted.h"
 #include "any_missing.h"
 #include "any_infinite.h"
 #include "all_missing.h"
-#include "all_nchar.h"
+#include "find_min_nchar.h"
 #include "helper.h"
 #include "guess_type.h"
 
@@ -30,6 +32,23 @@ static char msg[255] = "";
         } \
     }
 
+#define HANDLE_INTEGERISH_NULL(tol, null_ok) \
+    if (isNull((x))) { \
+        if (asFlag((null_ok), "null.ok")) \
+            return ScalarLogical(TRUE); \
+        snprintf(msg, 255, "Must be of type 'integerish', not 'NULL'"); \
+        return ScalarString(mkChar(msg)); \
+    } else { \
+        int_err_t ok = checkIntegerish(x, dtol, FALSE); \
+        switch(ok.err) { \
+            case INT_OK: break; \
+            case INT_TYPE: snprintf(msg, 255, "Must be of type 'integerish'%s, not '%s'", asFlag(null_ok, "null_ok") ? " (or 'NULL')" : "", guess_type(x)); return ScalarString(mkChar(msg)); \
+            case INT_RANGE: snprintf(msg, 255, "Must be of type 'integerish', but element %ld is not in integer range", ok.pos); return ScalarString(mkChar(msg)); \
+            case INT_TOL: snprintf(msg, 255, "Must be of type 'integerish', but element %ld is not close to an integer", ok.pos); return ScalarString(mkChar(msg)); \
+            case INT_COMPLEX: snprintf(msg, 255, "Must be of type 'integerish', but element %ld has an imaginary part", ok.pos); return ScalarString(mkChar(msg)); \
+        } \
+    }
+
 #define HANDLE_NA(x, na_ok) \
     if (is_scalar_na((x))) { \
         if (asFlag((na_ok), "na.ok")) \
@@ -62,10 +81,6 @@ static SEXP result(const char *fmt, ...) {
     return ScalarString(mkChar(msg));
 }
 
-static Rboolean is_posixct(SEXP x) {
-    return isReal(x) && inherits(x, "POSIXct");
-}
-
 static void fmt_posixct(char * out, SEXP x) {
     SEXP call = PROTECT(allocVector(LANGSXP, 2));
     SETCAR(call, install("format.POSIXct"));
@@ -73,44 +88,43 @@ static void fmt_posixct(char * out, SEXP x) {
     SEXP result = PROTECT(eval(call, R_GlobalEnv));
 
     strncpy(out, CHAR(STRING_ELT(result, 0)), 255);
+    out[255] = '\0';
     UNPROTECT(2);
 }
 
 static Rboolean check_bounds(SEXP x, SEXP lower, SEXP upper) {
     double tmp = asNumber(lower, "lower");
     if (R_FINITE(tmp)) {
+        const R_xlen_t n = length(x);
         if (isReal(x)) {
-            const double *xp = REAL(x);
-            const double * const xend = xp + xlength(x);
-            for (; xp != xend; xp++) {
-                if (!ISNAN(*xp) && *xp < tmp)
-                    return message("All elements must be >= %g", tmp);
+            const double *xp = REAL_RO(x);
+            for (R_xlen_t i = 0; i < n; i++) {
+                if (!ISNAN(xp[i]) && xp[i] < tmp)
+                    return message("Element %i is not >= %g", i + 1, tmp);
             }
         } else if (isInteger(x)) {
-            const int *xp = INTEGER(x);
-            const int * const xend = xp + xlength(x);
-            for (; xp != xend; xp++) {
-                if (*xp != NA_INTEGER && *xp < tmp)
-                    return message("All elements must be >= %g", tmp);
+            const int *xp = INTEGER_RO(x);
+            for (R_xlen_t i = 0; i < n; i++) {
+                if (xp[i] != NA_INTEGER && xp[i] < tmp)
+                    return message("Element %i is not >= %g", i + 1, tmp);
             }
         }
     }
 
     tmp = asNumber(upper, "upper");
     if (R_FINITE(tmp)) {
+        const R_xlen_t n = length(x);
         if (isReal(x)) {
-            const double *xp = REAL(x);
-            const double * const xend = xp + xlength(x);
-            for (; xp != xend; xp++) {
-                if (!ISNAN(*xp) && *xp > tmp)
-                    return message("All elements must be <= %g", tmp);
+            const double *xp = REAL_RO(x);
+            for (R_xlen_t i = 0; i < n; i++) {
+                if (!ISNAN(xp[i]) && xp[i] > tmp)
+                    return message("Element %i is not <= %g", i + 1, tmp);
             }
         } else if (isInteger(x)) {
-            const int *xp = INTEGER(x);
-            const int * const xend = xp + xlength(x);
-            for (; xp != xend; xp++) {
-                if (*xp != NA_INTEGER && *xp > tmp)
-                    return message("All elements must be <= %g", tmp);
+            const int *xp = INTEGER_RO(x);
+            for (R_xlen_t i = 0; i < n; i++) {
+                if (xp[i] != NA_INTEGER && xp[i] > tmp)
+                    return message("Element %i is not <= %g", i + 1, tmp);
             }
         }
     }
@@ -118,7 +132,7 @@ static Rboolean check_bounds(SEXP x, SEXP lower, SEXP upper) {
 }
 
 
-Rboolean check_posix_bounds(SEXP x, SEXP lower, SEXP upper) {
+static Rboolean check_posix_bounds(SEXP x, SEXP lower, SEXP upper) {
     if (isNull(lower) && isNull(upper))
         return TRUE;
 
@@ -126,7 +140,7 @@ Rboolean check_posix_bounds(SEXP x, SEXP lower, SEXP upper) {
     const Rboolean null_tz = isNull(tz);
 
     if (!isNull(lower)) {
-        if (!is_posixct(lower) || length(lower) != 1)
+        if (!is_class_posixct(lower) || length(lower) != 1)
             error("Argument 'lower' must be provided as single POSIXct time");
         SEXP lower_tz = PROTECT(getAttrib(lower, install("tzone")));
         if (null_tz != isNull(lower_tz) ||
@@ -135,22 +149,22 @@ Rboolean check_posix_bounds(SEXP x, SEXP lower, SEXP upper) {
             return message("Timezones of 'x' and 'lower' must match");
         }
 
-        const double tmp = REAL(lower)[0];
-        const double *xp = REAL(x);
-        const double * const xend = xp + xlength(x);
-        for (; xp != xend; xp++) {
-            if (!ISNAN(*xp) && *xp < tmp) {
-                char fmt[255];
+        const double tmp = REAL_RO(lower)[0];
+        const double *xp = REAL_RO(x);
+        const R_xlen_t n = length(x);
+        for (R_xlen_t i = 0; i < n; i++) {
+            if (!ISNAN(xp[i]) && xp[i] < tmp) {
+                char fmt[256];
                 fmt_posixct(fmt, lower);
                 UNPROTECT(2);
-                return message("All times must be >= %s", fmt);
+                return message("Element %i is not >= %s", i + 1, fmt);
             }
         }
         UNPROTECT(1);
     }
 
     if (!isNull(upper)) {
-        if (!is_posixct(upper) || length(upper) != 1)
+        if (!is_class_posixct(upper) || length(upper) != 1)
             error("Argument 'upper' must be provided as single POSIXct time");
         SEXP upper_tz = PROTECT(getAttrib(upper, install("tzone")));
         if (null_tz != isNull(upper_tz) ||
@@ -159,15 +173,15 @@ Rboolean check_posix_bounds(SEXP x, SEXP lower, SEXP upper) {
             return message("Timezones of 'x' and 'upper' must match");
         }
 
-        const double tmp = REAL(upper)[0];
-        const double *xp = REAL(x);
-        const double * const xend = xp + xlength(x);
-        for (; xp != xend; xp++) {
-            if (!ISNAN(*xp) && *xp > tmp) {
-                char fmt[255];
+        const double tmp = REAL_RO(upper)[0];
+        const double *xp = REAL_RO(x);
+        const R_xlen_t n = length(x);
+        for (R_xlen_t i = 0; i < n; i++) {
+            if (!ISNAN(xp[i]) && xp[i] > tmp) {
+                char fmt[256];
                 fmt_posixct(fmt, upper);
                 UNPROTECT(2);
-                return message("All times must be <= %s", fmt);
+                return message("Element %i is not <= %s", i + 1, fmt);
             }
         }
         UNPROTECT(1);
@@ -177,7 +191,7 @@ Rboolean check_posix_bounds(SEXP x, SEXP lower, SEXP upper) {
     return TRUE;
 }
 
-static Rboolean check_strict_names(SEXP x) {
+static R_xlen_t check_strict_names(SEXP x) {
     const R_xlen_t nx = xlength(x);
     const char *str;
     for (R_xlen_t i = 0; i < nx; i++) {
@@ -185,21 +199,21 @@ static Rboolean check_strict_names(SEXP x) {
         while (*str == '.')
             str++;
         if (!isalpha(*str))
-            return FALSE;
+            return i + 1;
         for (; *str != '\0'; str++) {
             if (!isalnum(*str) && *str != '.' && *str != '_')
-                return FALSE;
+                return i + 1;
         }
     }
-    return TRUE;
+    return 0;
 }
 
 static Rboolean check_names(SEXP nn, const char * type, const char * what) {
-    typedef enum { T_UNNAMED, T_NAMED, T_UNIQUE, T_STRICT } name_t;
+    typedef enum { T_UNNAMED, T_NAMED, T_UNIQUE, T_STRICT, T_IDS } name_t;
     name_t checks = T_UNNAMED;
 
     if (strcmp(type, "unnamed") == 0)
-        return isNull(nn) ? TRUE : message("%s must be unnamed, but has names", what);
+        return isNull(nn) ? TRUE : message("May not have %s", what);
 
     if (strcmp(type, "named") == 0) {
         checks = T_NAMED;
@@ -207,18 +221,36 @@ static Rboolean check_names(SEXP nn, const char * type, const char * what) {
         checks = T_UNIQUE;
     } else if (strcmp(type, "strict") == 0) {
         checks = T_STRICT;
+    } else if (strcmp(type, "ids") == 0) {
+        checks = T_IDS;
     } else {
         error("Unknown type '%s' to specify check for names. Supported are 'unnamed', 'named', 'unique' and 'strict'.", type);
     }
 
-    if (isNull(nn) || any_missing_string(nn) || !all_nchar(nn, 1, FALSE))
-        return message("%s must be named", what);
-    if (checks >= T_UNIQUE) {
-        if (any_duplicated(nn, FALSE) != 0)
-            return message("%s must be uniquely named", what);
-        if (checks >= T_STRICT && !check_strict_names(nn)) {
-            return message("%s must be named according to R's variable naming conventions and may not contain special characters", what);
-        }
+    if (isNull(nn)) {
+        return message("Must have %s", what);
+    }
+
+    R_xlen_t pos = find_missing_string(nn);
+    if (pos > 0) {
+        return message("Must have %s, but is NA at position %i", what, pos);
+    }
+
+    pos = find_min_nchar(nn, 1, FALSE);
+    if (pos > 0) {
+        return message("Must have %s, but element %i is empty", what, pos);
+    }
+
+    if (checks == T_UNIQUE || checks == T_STRICT) {
+        pos = any_duplicated(nn, FALSE);
+        if (pos > 0)
+            return message("Must have unique %s, but element %i is duplicated", what, pos);
+    }
+
+    if (checks == T_STRICT || checks == T_IDS) {
+        pos = check_strict_names(nn);
+        if (pos > 0)
+            return message("Must have %s according to R's variable naming conventions, but element %i does not comply", what, pos);
     }
     return TRUE;
 }
@@ -250,52 +282,69 @@ static Rboolean check_vector_len(SEXP x, SEXP len, SEXP min_len, SEXP max_len) {
 }
 
 static Rboolean check_vector_missings(SEXP x, SEXP any_missing, SEXP all_missing) {
-    if (!asFlag(any_missing, "any.missing") && any_missing_atomic(x))
-        return message("Contains missing values");
+    if (!asFlag(any_missing, "any.missing")) {
+        R_xlen_t pos = find_missing_vector(x);
+        if (pos > 0)
+            return message("Contains missing values (element %i)", pos);
+    }
     if (!asFlag(all_missing, "all.missing") && all_missing_atomic(x))
         return message("Contains only missing values");
     return TRUE;
 }
 
 static Rboolean check_vector_unique(SEXP x, SEXP unique) {
-    if (asFlag(unique, "unique") && any_duplicated(x, FALSE) > 0)
-        return message("Contains duplicated values");
+    if (asFlag(unique, "unique")) {
+        R_xlen_t pos = any_duplicated(x, FALSE);
+        if (pos > 0)
+            return message("Contains duplicated values, position %i", pos);
+    }
     return TRUE;
 }
 
 static Rboolean check_vector_names(SEXP x, SEXP names) {
     if (!isNull(names) && xlength(x) > 0)
-        return check_named(x, asString(names, "names"), "Vector");
+        return check_named(x, asString(names, "names"), "names");
     return TRUE;
 }
 
 static Rboolean check_vector_finite(SEXP x, SEXP finite) {
+    // FIXME: pos
     if (asFlag(finite, "finite") && any_infinite(x))
         return message("Must be finite");
     return TRUE;
 }
 
-static Rboolean check_matrix_dims(SEXP x, SEXP min_rows, SEXP min_cols, SEXP rows, SEXP cols) {
-    if (!isNull(min_rows) || !isNull(rows)) {
+static Rboolean check_matrix_dims(SEXP x, SEXP min_rows, SEXP max_rows, SEXP min_cols, SEXP max_cols, SEXP rows, SEXP cols) {
+    if (!isNull(min_rows) || !isNull(max_rows) || !isNull(rows)) {
         R_len_t xrows = get_nrows(x);
         if (!isNull(min_rows)) {
             R_len_t cmp = asLength(min_rows, "min.rows");
             if (xrows < cmp)
                 return message("Must have at least %i rows, but has %i rows", cmp, xrows);
         }
+        if (!isNull(max_rows)) {
+            R_len_t cmp = asLength(max_rows, "max.rows");
+            if (xrows > cmp)
+                return message("Must have at most %i rows, but has %i rows", cmp, xrows);
+        }
         if (!isNull(rows)) {
             R_len_t cmp = asLength(rows, "rows");
             if (xrows != cmp)
                 return message("Must have exactly %i rows, but has %i rows", cmp, xrows);
         }
     }
-    if (!isNull(min_cols) || !isNull(cols)) {
+    if (!isNull(min_cols) || !isNull(max_cols) || !isNull(cols)) {
         R_len_t xcols = get_ncols(x);
         if (!isNull(min_cols)) {
             R_len_t cmp = asLength(min_cols, "min.cols");
             if (xcols < cmp)
                 return message("Must have at least %i cols, but has %i cols", cmp, xcols);
         }
+        if (!isNull(max_cols)) {
+            R_len_t cmp = asLength(max_cols, "max.cols");
+            if (xcols > cmp)
+                return message("Must have at most %i cols, but has %i cols", cmp, xcols);
+        }
         if (!isNull(cols)) {
             R_len_t cmp = asCount(cols, "cols");
             if (xcols != cmp)
@@ -321,7 +370,7 @@ static Rboolean check_storage(SEXP x, SEXP mode) {
             if (!isIntegerish(x, INTEGERISH_DEFAULT_TOL, FALSE))
                 return message("Must store integerish values");
         } else if (strcmp(storage, "numeric") == 0) {
-            if (!isStrictlyNumeric(x))
+            if (!is_class_numeric(x))
                 return message("Must store numerics");
         } else if (strcmp(storage, "complex") == 0) {
             if (!isComplex(x))
@@ -330,7 +379,7 @@ static Rboolean check_storage(SEXP x, SEXP mode) {
             if (!isString(x))
                 return message("Must store characters");
         } else if (strcmp(storage, "list") == 0) {
-            if (!isRList(x))
+            if (!is_class_list(x))
                 return message("Must store a list");
         } else if (strcmp(storage, "atomic") == 0) {
             if (!isVectorAtomic(x))
@@ -345,66 +394,18 @@ static Rboolean check_storage(SEXP x, SEXP mode) {
 static inline Rboolean is_scalar_na(SEXP x) {
     if (xlength(x) == 1) {
         switch(TYPEOF(x)) {
-            case LGLSXP: return (LOGICAL(x)[0] == NA_LOGICAL);
-            case INTSXP: return (INTEGER(x)[0] == NA_INTEGER);
-            case REALSXP: return ISNAN(REAL(x)[0]);
+            case LGLSXP: return (LOGICAL_RO(x)[0] == NA_LOGICAL);
+            case INTSXP: return (INTEGER_RO(x)[0] == NA_INTEGER);
+            case REALSXP: return ISNAN(REAL_RO(x)[0]);
             case STRSXP: return (STRING_ELT(x, 0) == NA_STRING);
         }
     }
     return FALSE;
 }
 
-static Rboolean is_sorted_integer(SEXP x) {
-    R_xlen_t i = 0;
-    const int * const xi = INTEGER(x);
-    const R_xlen_t n = xlength(x);
-    while(xi[i] == NA_INTEGER) {
-        i++;
-        if (i == n)
-            return TRUE;
-    }
-
-    for (R_xlen_t j = i + 1; j < n; j++) {
-        if (xi[j] != NA_INTEGER) {
-            if (xi[i] > xi[j])
-                return FALSE;
-            i = j;
-        }
-    }
-    return TRUE;
-}
-
-
-static Rboolean is_sorted_double(SEXP x) {
-    R_xlen_t i = 0;
-    const double * const xr = REAL(x);
-    const R_xlen_t n = xlength(x);
-    while(xr[i] == NA_REAL) {
-        i++;
-        if (i == n)
-            return TRUE;
-    }
-
-    for (R_xlen_t j = i + 1; j < n; j++) {
-        if (xr[j] != NA_REAL) {
-            if (xr[i] > xr[j])
-                return FALSE;
-            i = j;
-        }
-    }
-    return TRUE;
-}
-
-
 static Rboolean check_vector_sorted(SEXP x, SEXP sorted) {
     if (asFlag(sorted, "sorted") && xlength(x) > 1) {
-        Rboolean ok;
-        switch(TYPEOF(x)) {
-            case INTSXP: ok = is_sorted_integer(x); break;
-            case REALSXP: ok = is_sorted_double(x); break;
-            default: error("Checking for sorted vector only possible for integer and double");
-        }
-        if (!ok)
+        if (!isSorted(x))
             return message("Must be sorted");
     }
     return TRUE;
@@ -413,22 +414,23 @@ static Rboolean check_vector_sorted(SEXP x, SEXP sorted) {
 /*********************************************************************************************************************/
 /* Exported check functions                                                                                          */
 /*********************************************************************************************************************/
-SEXP attribute_hidden c_check_character(SEXP x, SEXP min_chars, SEXP any_missing, SEXP all_missing, SEXP len, SEXP min_len, SEXP max_len, SEXP unique, SEXP names, SEXP null_ok) {
-    HANDLE_TYPE_NULL(isString(x) || all_missing_atomic(x), "character", null_ok);
+SEXP attribute_hidden c_check_character(SEXP x, SEXP min_chars, SEXP any_missing, SEXP all_missing, SEXP len, SEXP min_len, SEXP max_len, SEXP unique, SEXP sorted, SEXP names, SEXP null_ok) {
+    HANDLE_TYPE_NULL(is_class_string(x) || all_missing_atomic(x), "character", null_ok);
     ASSERT_TRUE(check_vector_len(x, len, min_len, max_len));
     ASSERT_TRUE(check_vector_names(x, names));
     ASSERT_TRUE(check_vector_missings(x, any_missing, all_missing));
     if (!isNull(min_chars)) {
         R_xlen_t n = asCount(min_chars, "min.chars");
-        if (n > 0 && !all_nchar(x, n, TRUE))
+        if (n > 0 && find_min_nchar(x, n, TRUE) > 0)
             return result("All elements must have at least %i characters", n);
     }
     ASSERT_TRUE(check_vector_unique(x, unique));
+    ASSERT_TRUE(check_vector_sorted(x, sorted));
     return ScalarLogical(TRUE);
 }
 
 SEXP attribute_hidden c_check_complex(SEXP x, SEXP any_missing, SEXP all_missing, SEXP len, SEXP min_len, SEXP max_len, SEXP unique, SEXP names, SEXP null_ok) {
-    HANDLE_TYPE_NULL(isComplex(x) || all_missing_atomic(x), "complex", null_ok);
+    HANDLE_TYPE_NULL(is_class_complex(x) || all_missing_atomic(x), "complex", null_ok);
     ASSERT_TRUE(check_vector_len(x, len, min_len, max_len));
     ASSERT_TRUE(check_vector_names(x, names));
     ASSERT_TRUE(check_vector_missings(x, any_missing, all_missing));
@@ -436,28 +438,41 @@ SEXP attribute_hidden c_check_complex(SEXP x, SEXP any_missing, SEXP all_missing
     return ScalarLogical(TRUE);
 }
 
-SEXP attribute_hidden c_check_dataframe(SEXP x, SEXP any_missing, SEXP all_missing, SEXP min_rows, SEXP min_cols, SEXP rows, SEXP cols, SEXP row_names, SEXP col_names, SEXP null_ok) {
-    HANDLE_TYPE_NULL(isFrame(x), "data.frame", null_ok);
-    ASSERT_TRUE(check_matrix_dims(x, min_rows, min_cols, rows, cols));
+SEXP attribute_hidden c_check_dataframe(SEXP x, SEXP any_missing, SEXP all_missing, SEXP min_rows, SEXP max_rows, SEXP min_cols, SEXP max_cols, SEXP rows, SEXP cols, SEXP row_names, SEXP col_names, SEXP null_ok) {
+    HANDLE_TYPE_NULL(is_class_frame(x), "data.frame", null_ok);
+    ASSERT_TRUE(check_matrix_dims(x, min_rows, max_rows, min_cols, max_cols, rows, cols));
 
     if (!isNull(row_names)) {
         SEXP nn = PROTECT(getAttrib(x, install("row.names")));
-        if (isInteger(nn))
-            nn = coerceVector(nn, STRSXP);
-        ASSERT_TRUE_UNPROTECT(check_names(nn, asString(row_names, "row.names"), "Rows"), 1);
+        int nprotect = 1;
+        if (isInteger(nn)) {
+            nn = PROTECT(coerceVector(nn, STRSXP));
+            nprotect++;
+        }
+        ASSERT_TRUE_UNPROTECT(check_names(nn, asString(row_names, "row.names"), "rownames"), nprotect);
     }
 
-    if (!isNull(col_names))
-        ASSERT_TRUE(check_named(x, asString(col_names, "col.names"), "Columns"));
-    if (!asFlag(any_missing, "any.missing") && any_missing_frame(x))
-        return result("Contains missing values");
-    if (!asFlag(all_missing, "all.missing") && all_missing_frame(x))
+    if (!isNull(col_names)) {
+        ASSERT_TRUE(check_named(x, asString(col_names, "col.names"), "colnames"));
+    }
+    if (!asFlag(any_missing, "any.missing")) {
+        R_xlen_t pos = find_missing_frame(x);
+        if (pos > 0) {
+            R_xlen_t nrow = get_nrows(x);
+            const char * nn = CHAR(STRING_ELT(getAttrib(x, R_NamesSymbol), translate_col(pos, nrow)));
+            return result("Contains missing values (column '%s', row %i)",
+                    nn, translate_row(pos, nrow) + 1);
+        }
+    }
+
+    if (!asFlag(all_missing, "all.missing") && all_missing_frame(x)) {
         return result("Contains only missing values");
+    }
     return ScalarLogical(TRUE);
 }
 
 SEXP attribute_hidden c_check_factor(SEXP x, SEXP any_missing, SEXP all_missing, SEXP len, SEXP min_len, SEXP max_len, SEXP unique, SEXP names, SEXP null_ok) {
-    HANDLE_TYPE_NULL(isFactor(x) || all_missing_atomic(x), "factor", null_ok);
+    HANDLE_TYPE_NULL(is_class_factor(x) || all_missing_atomic(x), "factor", null_ok);
     ASSERT_TRUE(check_vector_len(x, len, min_len, max_len));
     ASSERT_TRUE(check_vector_names(x, names));
     ASSERT_TRUE(check_vector_missings(x, any_missing, all_missing));
@@ -466,7 +481,7 @@ SEXP attribute_hidden c_check_factor(SEXP x, SEXP any_missing, SEXP all_missing,
 }
 
 SEXP attribute_hidden c_check_integer(SEXP x, SEXP lower, SEXP upper, SEXP any_missing, SEXP all_missing, SEXP len, SEXP min_len, SEXP max_len, SEXP unique, SEXP sorted, SEXP names, SEXP null_ok) {
-    HANDLE_TYPE_NULL(isInteger(x) || all_missing_atomic(x), "integer", null_ok);
+    HANDLE_TYPE_NULL(is_class_integer(x) || all_missing_atomic(x), "integer", null_ok);
     ASSERT_TRUE(check_vector_len(x, len, min_len, max_len));
     ASSERT_TRUE(check_vector_names(x, names));
     ASSERT_TRUE(check_vector_missings(x, any_missing, all_missing));
@@ -478,7 +493,7 @@ SEXP attribute_hidden c_check_integer(SEXP x, SEXP lower, SEXP upper, SEXP any_m
 
 SEXP attribute_hidden c_check_integerish(SEXP x, SEXP tol, SEXP lower, SEXP upper, SEXP any_missing, SEXP all_missing, SEXP len, SEXP min_len, SEXP max_len, SEXP unique, SEXP sorted, SEXP names, SEXP null_ok) {
     double dtol = asNumber(tol, "tol");
-    HANDLE_TYPE_NULL(isIntegerish(x, dtol, FALSE) || all_missing_atomic(x), "integerish", null_ok);
+    HANDLE_INTEGERISH_NULL(dtol, null_ok);
     ASSERT_TRUE(check_vector_len(x, len, min_len, max_len));
     ASSERT_TRUE(check_vector_names(x, names));
     ASSERT_TRUE(check_vector_missings(x, any_missing, all_missing));
@@ -489,7 +504,7 @@ SEXP attribute_hidden c_check_integerish(SEXP x, SEXP tol, SEXP lower, SEXP uppe
 }
 
 SEXP attribute_hidden c_check_list(SEXP x, SEXP any_missing, SEXP all_missing, SEXP len, SEXP min_len, SEXP max_len, SEXP unique, SEXP names, SEXP null_ok) {
-    HANDLE_TYPE_NULL(isRList(x), "list", null_ok)
+    HANDLE_TYPE_NULL(is_class_list(x), "list", null_ok)
     ASSERT_TRUE(check_vector_len(x, len, min_len, max_len));
     ASSERT_TRUE(check_vector_names(x, names));
     ASSERT_TRUE(check_vector_missings(x, any_missing, all_missing));
@@ -498,7 +513,7 @@ SEXP attribute_hidden c_check_list(SEXP x, SEXP any_missing, SEXP all_missing, S
 }
 
 SEXP attribute_hidden c_check_logical(SEXP x, SEXP any_missing, SEXP all_missing, SEXP len, SEXP min_len, SEXP max_len, SEXP unique, SEXP names, SEXP null_ok) {
-    HANDLE_TYPE_NULL(isLogical(x) || all_missing_atomic(x), "logical", null_ok);
+    HANDLE_TYPE_NULL(is_class_logical(x) || all_missing_atomic(x), "logical", null_ok);
     ASSERT_TRUE(check_vector_len(x, len, min_len, max_len));
     ASSERT_TRUE(check_vector_names(x, names));
     ASSERT_TRUE(check_vector_missings(x, any_missing, all_missing));
@@ -506,33 +521,45 @@ SEXP attribute_hidden c_check_logical(SEXP x, SEXP any_missing, SEXP all_missing
     return ScalarLogical(TRUE);
 }
 
-SEXP attribute_hidden c_check_matrix(SEXP x, SEXP mode, SEXP any_missing, SEXP all_missing, SEXP min_rows, SEXP min_cols, SEXP rows, SEXP cols, SEXP row_names, SEXP col_names, SEXP null_ok) {
-    HANDLE_TYPE_NULL(isMatrix(x), "matrix", null_ok);
+SEXP attribute_hidden c_check_matrix(SEXP x, SEXP mode, SEXP any_missing, SEXP all_missing, SEXP min_rows, SEXP max_rows, SEXP min_cols, SEXP max_cols, SEXP rows, SEXP cols, SEXP row_names, SEXP col_names, SEXP null_ok) {
+    HANDLE_TYPE_NULL(is_class_matrix(x), "matrix", null_ok);
     ASSERT_TRUE(check_storage(x, mode));
-    ASSERT_TRUE(check_matrix_dims(x, min_rows, min_cols, rows, cols));
+    ASSERT_TRUE(check_matrix_dims(x, min_rows, max_rows, min_cols, max_cols, rows, cols));
 
     if (!isNull(row_names) && xlength(x) > 0) {
         SEXP nn = PROTECT(getAttrib(x, R_DimNamesSymbol));
         if (!isNull(nn))
             nn = VECTOR_ELT(nn, 0);
-        ASSERT_TRUE_UNPROTECT(check_names(nn, asString(row_names, "row.names"), "Rows"), 1);
+        ASSERT_TRUE_UNPROTECT(check_names(nn, asString(row_names, "row.names"), "rownames"), 1);
     }
 
     if (!isNull(col_names) && xlength(x) > 0) {
         SEXP nn = PROTECT(getAttrib(x, R_DimNamesSymbol));
         if (!isNull(nn))
             nn = VECTOR_ELT(nn, 1);
-        ASSERT_TRUE_UNPROTECT(check_names(nn, asString(col_names, "col.names"), "Columns"), 1);
+        ASSERT_TRUE_UNPROTECT(check_names(nn, asString(col_names, "col.names"), "colnames"), 1);
+    }
+
+    if (!asFlag(any_missing, "any.missing")) {
+        R_xlen_t pos = find_missing_matrix(x);
+        if (pos > 0) {
+            R_xlen_t nrow = get_nrows(x);
+            return result("Contains missing values (row %i, col %i)",
+                    translate_row(pos, nrow) + 1, translate_col(pos, nrow) + 1);
+        }
+    }
+
+    if (!asFlag(all_missing, "all.missing") && all_missing_atomic(x)) {
+        return result("Contains only missing values");
     }
-    ASSERT_TRUE(check_vector_missings(x, any_missing, all_missing));
     return ScalarLogical(TRUE);
 }
 
 SEXP attribute_hidden c_check_array(SEXP x, SEXP mode, SEXP any_missing, SEXP d, SEXP min_d, SEXP max_d, SEXP null_ok) {
-    HANDLE_TYPE_NULL(isArray(x), "array", null_ok);
+    HANDLE_TYPE_NULL(is_class_array(x), "array", null_ok);
     ASSERT_TRUE(check_storage(x, mode));
 
-    if (!asFlag(any_missing, "any.missing") && any_missing_atomic(x))
+    if (!asFlag(any_missing, "any.missing") && find_missing_vector(x) > 0)
         return result("Contains missing values");
 
     R_len_t ndim = length(getAttrib(x, R_DimSymbol));
@@ -563,16 +590,27 @@ SEXP attribute_hidden c_check_named(SEXP x, SEXP type) {
     return ScalarLogical(TRUE);
 }
 
-SEXP attribute_hidden c_check_names(SEXP x, SEXP type) {
+SEXP attribute_hidden c_check_names(SEXP x, SEXP type, SEXP what) {
     if (!(isString(x) || isNull(x)))
-        return result("Must be a character vector of names");
-    ASSERT_TRUE(check_names(x, asString(type, "type"), "Names"));
+        return result("Must be a character vector");
+    ASSERT_TRUE(check_names(x, asString(type, "type"), asString(what, "what")));
     return ScalarLogical(TRUE);
 }
 
-
 SEXP attribute_hidden c_check_numeric(SEXP x, SEXP lower, SEXP upper, SEXP finite, SEXP any_missing, SEXP all_missing, SEXP len, SEXP min_len, SEXP max_len, SEXP unique, SEXP sorted, SEXP names, SEXP null_ok) {
-    HANDLE_TYPE_NULL(isStrictlyNumeric(x) || all_missing_atomic(x), "numeric", null_ok);
+    HANDLE_TYPE_NULL(is_class_numeric(x) || all_missing_atomic(x), "numeric", null_ok);
+    ASSERT_TRUE(check_vector_len(x, len, min_len, max_len));
+    ASSERT_TRUE(check_vector_names(x, names));
+    ASSERT_TRUE(check_vector_missings(x, any_missing, all_missing));
+    ASSERT_TRUE(check_bounds(x, lower, upper));
+    ASSERT_TRUE(check_vector_finite(x, finite));
+    ASSERT_TRUE(check_vector_unique(x, unique));
+    ASSERT_TRUE(check_vector_sorted(x, sorted));
+    return ScalarLogical(TRUE);
+}
+
+SEXP attribute_hidden c_check_double(SEXP x, SEXP lower, SEXP upper, SEXP finite, SEXP any_missing, SEXP all_missing, SEXP len, SEXP min_len, SEXP max_len, SEXP unique, SEXP sorted, SEXP names, SEXP null_ok) {
+    HANDLE_TYPE_NULL(is_class_double(x) || all_missing_atomic(x), "double", null_ok);
     ASSERT_TRUE(check_vector_len(x, len, min_len, max_len));
     ASSERT_TRUE(check_vector_names(x, names));
     ASSERT_TRUE(check_vector_missings(x, any_missing, all_missing));
@@ -596,8 +634,15 @@ SEXP attribute_hidden c_check_vector(SEXP x, SEXP strict, SEXP any_missing, SEXP
     return ScalarLogical(TRUE);
 }
 
+SEXP attribute_hidden c_check_raw(SEXP x, SEXP len, SEXP min_len, SEXP max_len, SEXP names, SEXP null_ok) {
+    HANDLE_TYPE_NULL(is_class_raw(x), "raw", null_ok);
+    ASSERT_TRUE(check_vector_len(x, len, min_len, max_len));
+    ASSERT_TRUE(check_vector_names(x, names));
+    return ScalarLogical(TRUE);
+}
+
 SEXP attribute_hidden c_check_atomic(SEXP x, SEXP any_missing, SEXP all_missing, SEXP len, SEXP min_len, SEXP max_len, SEXP unique, SEXP names) {
-    HANDLE_TYPE(isNull(x) || isVectorAtomic(x), "atomic");
+    HANDLE_TYPE(is_class_atomic(x), "atomic");
     ASSERT_TRUE(check_vector_len(x, len, min_len, max_len));
     ASSERT_TRUE(check_vector_names(x, names));
     ASSERT_TRUE(check_vector_missings(x, any_missing, all_missing));
@@ -606,7 +651,7 @@ SEXP attribute_hidden c_check_atomic(SEXP x, SEXP any_missing, SEXP all_missing,
 }
 
 SEXP attribute_hidden c_check_atomic_vector(SEXP x, SEXP any_missing, SEXP all_missing, SEXP len, SEXP min_len, SEXP max_len, SEXP unique, SEXP names) {
-    HANDLE_TYPE(isAtomicVector(x), "atomic vector");
+    HANDLE_TYPE(is_class_atomic_vector(x), "atomic vector");
     ASSERT_TRUE(check_vector_len(x, len, min_len, max_len));
     ASSERT_TRUE(check_vector_names(x, names));
     ASSERT_TRUE(check_vector_missings(x, any_missing, all_missing));
@@ -646,7 +691,7 @@ SEXP attribute_hidden c_check_int(SEXP x, SEXP na_ok, SEXP lower, SEXP upper, SE
 
 SEXP attribute_hidden c_check_number(SEXP x, SEXP na_ok, SEXP lower, SEXP upper, SEXP finite, SEXP null_ok) {
     HANDLE_NA(x, na_ok);
-    HANDLE_TYPE_NULL(isStrictlyNumeric(x), "number", null_ok);
+    HANDLE_TYPE_NULL(is_class_numeric(x), "number", null_ok);
     if (xlength(x) != 1)
         return result("Must have length 1");
     ASSERT_TRUE(check_vector_finite(x, finite));
@@ -661,7 +706,7 @@ SEXP attribute_hidden c_check_string(SEXP x, SEXP na_ok, SEXP min_chars, SEXP nu
         return result("Must have length 1");
     if (!isNull(min_chars)) {
         R_xlen_t n = asCount(min_chars, "min.chars");
-        if (!all_nchar(x, n, TRUE))
+        if (find_min_nchar(x, n, TRUE) > 0)
             return result("Must have at least %i characters", n);
     }
 
@@ -677,7 +722,7 @@ SEXP attribute_hidden c_check_scalar(SEXP x, SEXP na_ok, SEXP null_ok) {
 }
 
 SEXP attribute_hidden c_check_posixct(SEXP x, SEXP lower, SEXP upper, SEXP any_missing, SEXP all_missing, SEXP len, SEXP min_len, SEXP max_len, SEXP unique, SEXP sorted, SEXP null_ok) {
-    HANDLE_TYPE_NULL(is_posixct(x), "POSIXct", null_ok);
+    HANDLE_TYPE_NULL(is_class_posixct(x), "POSIXct", null_ok);
     ASSERT_TRUE(check_vector_len(x, len, min_len, max_len));
     ASSERT_TRUE(check_vector_missings(x, any_missing, all_missing));
     ASSERT_TRUE(check_vector_unique(x, unique));
