@@ -23,9 +23,9 @@ extern SEXP c_check_integer(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP
 extern SEXP c_check_integerish(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP c_check_list(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP c_check_logical(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP c_check_matrix(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+extern SEXP c_check_matrix(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP c_check_named(SEXP, SEXP);
-extern SEXP c_check_names(SEXP, SEXP);
+extern SEXP c_check_names(SEXP, SEXP, SEXP);
 extern SEXP c_check_number(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP c_check_numeric(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP c_check_double(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
@@ -62,7 +62,7 @@ static const R_CallMethodDef CallEntries[] = {
     {"c_check_logical",       (DL_FUNC) &c_check_logical,        9},
     {"c_check_matrix",        (DL_FUNC) &c_check_matrix,        13},
     {"c_check_named",         (DL_FUNC) &c_check_named,          2},
-    {"c_check_names",         (DL_FUNC) &c_check_names,          2},
+    {"c_check_names",         (DL_FUNC) &c_check_names,          3},
     {"c_check_number",        (DL_FUNC) &c_check_number,         6},
     {"c_check_numeric",       (DL_FUNC) &c_check_numeric,       13},
     {"c_check_double",        (DL_FUNC) &c_check_double,        13},
